import os
import time
import signal
from subprocess import check_output, Popen

import jujuresources
from charmhelpers.core import hookenv
from charmhelpers.core import unitdata
from jujubigdata import utils


class Hive(object):
    HIVE_VERSION = {'x86_64': '1.0.0', 'ppc64le': '0.13.0'}

    def __init__(self, dist_config):
        self.dist_config = dist_config
        self.cpu_arch = check_output(['uname', '-p']).strip()

    def is_installed(self):
        return unitdata.kv().get('hive.installed')

    def install(self, force=False):
        if not force and self.is_installed():
            return
        mirror_url = hookenv.config()['resources_mirror']
        jujuresources.fetch('hive-%s' % self.cpu_arch, mirror_url=mirror_url)
        jujuresources.install('hive-%s' % self.cpu_arch,
                              destination=self.dist_config.path('hive'),
                              skip_top_level=True)
        self.dist_config.add_users()
        self.dist_config.add_dirs()
        self.dist_config.add_packages()
        self.setup_hive_config()
        self.configure_hive()
        unitdata.kv().set('hive.installed', True)
        unitdata.kv().flush(True)

    def setup_hive_config(self):
        # copy default config into alternate dir
        conf_dir = self.dist_config.path('hive') / 'conf'
        self.dist_config.path('hive_conf').rmtree_p()
        conf_dir.copytree(self.dist_config.path('hive_conf'))
        hive_env = self.dist_config.path('hive_conf') / 'hive-env.sh'
        if not hive_env.exists():
            (self.dist_config.path('hive_conf') / 'hive-env.sh.template').copy(hive_env)
        hive_site = self.dist_config.path('hive_conf') / 'hive-site.xml'
        if not hive_site.exists():
            (self.dist_config.path('hive_conf') / 'hive-default.xml.template').copy(hive_site)

    def configure_hive(self):
        config = hookenv.config()
        hive_bin = self.dist_config.path('hive') / 'bin'
        with utils.environment_edit_in_place('/etc/environment') as env:
            if hive_bin not in env['PATH']:
                env['PATH'] = ':'.join([env['PATH'], hive_bin])
            env['HIVE_CONF_DIR'] = self.dist_config.path('hive_conf')
        hive_site = self.dist_config.path('hive_conf') / 'hive-site.xml'
        with utils.xmlpropmap_edit_in_place(hive_site) as props:
            mysql = unitdata.kv().get('relations.ready')['db'].values()[0]
            props['javax.jdo.option.ConnectionURL'] = "jdbc:mysql://{}/{}".format(
                mysql['host'], mysql['database']
            )
            # TODO (kwm): we should be able to export java.io.tmpdir so these 4 arent needed
            props['hive.exec.local.scratchdir'] = "/tmp/hive"
            props['hive.downloaded.resources.dir'] = "/tmp/hive_resources"
            props['hive.querylog.location'] = "/tmp/hive"
            props['hive.server2.logging.operation.log.location'] = "/tmp/hive"
            ####
            props['javax.jdo.option.ConnectionUserName'] = mysql['user']
            props['javax.jdo.option.ConnectionPassword'] = mysql['password']
            props['javax.jdo.option.ConnectionDriverName'] = "com.mysql.jdbc.Driver"
            props['hive.hwi.war.file'] = "lib/hive-hwi-%s.jar" % self.HIVE_VERSION[self.cpu_arch]

        hive_env = self.dist_config.path('hive_conf') / 'hive-env.sh'
        utils.re_edit_in_place(hive_env, {
            r'.*export HADOOP_HEAPSIZE *=.*': 'export HADOOP_HEAPSIZE=%s' % config['heap'],
            r'.*export HIVE_AUX_JARS_PATH *=.*': 'export HIVE_AUX_JARS_PATH=/usr/share/java/mysql-connector-java.jar',
        })

        utils.run_as('hive', 'hdfs', 'dfs', '-mkdir', '-p', '/user/hive/warehouse')
        utils.run_as('hive', 'schematool', '-initSchema', '-dbType', 'mysql')

    def run_bg(self, user, command, *args):
        """
        Run a Hive command as the `hive` user in the background.

        :param str command: Command to run
        :param list args: Additional args to pass to the command
        """
        parts = [command] + list(args)
        quoted = ' '.join("'%s'" % p for p in parts)
        Popen(['su', user, '-c', quoted])

    def start(self):
        if not utils.jps('HiveServer2'):
            self.run_bg(
                'hive', 'hive',
                '--config', self.dist_config.path('hive_conf'),
                '--service', 'hiveserver2')
            time.sleep(5)

    def stop(self):
        hive_pids = utils.jps('HiveServer2')
        for pid in hive_pids:
            os.kill(int(pid), signal.SIGTERM)

    def cleanup(self):
        self.dist_config.remove_users()
        self.dist_config.remove_dirs()
